@@ -32,6 +32,7 @@
 #include "ephy-encodings.h"
 #include "ephy-file-helpers.h"
 #include "ephy-filters-manager.h"
+#include "ephy-flatpak-utils.h"
 #include "ephy-history-service.h"
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
@@ -707,6 +708,7 @@ initialize_web_extensions (WebKitWebContext *web_context,
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   GVariant *user_data;
   gboolean private_profile;
+  gboolean browser_mode;
   const char *address;
 
   webkit_web_context_set_web_extensions_directory (web_context, EPHY_WEB_EXTENSIONS_DIR);
@@ -714,11 +716,13 @@ initialize_web_extensions (WebKitWebContext *web_context,
   address = priv->dbus_server ? g_dbus_server_get_client_address (priv->dbus_server) : NULL;
 
   private_profile = priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE || priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO;
-  user_data = g_variant_new ("(msssb)",
+  browser_mode = priv->mode == EPHY_EMBED_SHELL_MODE_BROWSER;
+  user_data = g_variant_new ("(msssbb)",
                              address,
                              ephy_dot_dir (),
                              ephy_filters_manager_get_adblock_filters_dir (priv->filters_manager),
-                             private_profile);
+                             private_profile,
+                             browser_mode);
   webkit_web_context_set_web_extensions_initialization_user_data (web_context, user_data);
 }
 
@@ -927,10 +931,6 @@ ephy_embed_shell_startup (GApplication *application)
 
   G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->startup (application);
 
-  filters_dir = adblock_filters_dir (shell);
-  priv->filters_manager = ephy_filters_manager_new (filters_dir);
-  g_free (filters_dir);
-
   ephy_embed_shell_create_web_context (shell);
 
   ephy_embed_shell_setup_web_extensions_server (shell);
@@ -1026,6 +1026,10 @@ ephy_embed_shell_startup (GApplication *application)
   ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, cookie_policy);
   g_free (cookie_policy);
 
+  filters_dir = adblock_filters_dir (shell);
+  priv->filters_manager = ephy_filters_manager_new (filters_dir);
+  g_free (filters_dir);
+
 #if ENABLE_HTTPS_EVERYWHERE
     /* We might want to be smarter about this in the future. For now,
      * trigger an update of the rulesets once each time Epiphany is started.
@@ -1441,6 +1445,10 @@ ephy_embed_shell_launch_handler (EphyEmbedShell *shell,
   g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), FALSE);
   g_return_val_if_fail (file || mime_type, FALSE);
 
+  if (ephy_is_running_inside_flatpak ()) {
+    return ephy_file_launch_file_via_uri_handler (file);
+  }
+
   app = ephy_file_launcher_get_app_info_for_file (file, mime_type);
 
   /* Do not allow recursive calls into the browser, they can lead to
