@@ -128,7 +128,8 @@ ephy_embed_utils_address_has_web_scheme (const char *address)
                      g_ascii_strncasecmp (address, "data", colonpos) &&
                      g_ascii_strncasecmp (address, "about", colonpos) &&
                      g_ascii_strncasecmp (address, "ephy-about", colonpos) &&
-                     g_ascii_strncasecmp (address, "gopher", colonpos));
+                     g_ascii_strncasecmp (address, "gopher", colonpos) &&
+                     g_ascii_strncasecmp (address, "inspector", colonpos));
 
   return has_web_scheme;
 }
@@ -276,7 +277,7 @@ ephy_embed_utils_autosearch_address (const char *search_key)
 
   shell = ephy_embed_shell_get_default ();
   search_engine_manager = ephy_embed_shell_get_search_engine_manager (shell);
-  address_search = ephy_search_engine_manager_get_address_for_default_engine (search_engine_manager);
+  address_search = ephy_search_engine_manager_get_default_search_address (search_engine_manager);
 
   query_param = soup_form_encode ("q", search_key, NULL);
 #pragma GCC diagnostic push
