@@ -1375,11 +1375,18 @@ ephy_sync_service_sync_collection (EphySyncService           *self,
 static gboolean
 ephy_sync_service_sync_internal (EphySyncService *self)
 {
+  GNetworkMonitor *monitor;
   guint index = 0;
   guint num_managers;
 
   g_assert (ephy_sync_utils_user_is_signed_in ());
 
+  monitor = g_network_monitor_get_default ();
+  if (g_network_monitor_get_connectivity (monitor) != G_NETWORK_CONNECTIVITY_FULL) {
+    g_signal_emit (self, signals[SYNC_FINISHED], 0);
+    return G_SOURCE_CONTINUE;
+  }
+
   if (!self->managers) {
     g_signal_emit (self, signals[SYNC_FINISHED], 0);
     return G_SOURCE_CONTINUE;
@@ -1911,6 +1918,7 @@ ephy_sync_service_upload_meta_global (EphySyncService *self)
   json_object_set_object_member (engines, "bookmarks", make_engine_object (2));
   json_object_set_object_member (engines, "history", make_engine_object (1));
   json_object_set_object_member (engines, "passwords", make_engine_object (1));
+  json_object_set_object_member (engines, "tabs", make_engine_object (1));
   json_object_set_object_member (engines, "forms", make_engine_object (1));
   json_object_set_object_member (payload, "engines", engines);
   json_object_set_int_member (payload, "storageVersion", EPHY_STORAGE_VERSION);
@@ -2190,10 +2198,16 @@ synchronizable_deleted_cb (EphySynchronizableManager *manager,
                            EphySynchronizable        *synchronizable,
                            EphySyncService           *self)
 {
+  GNetworkMonitor *monitor;
+
   g_assert (EPHY_IS_SYNCHRONIZABLE_MANAGER (manager));
   g_assert (EPHY_IS_SYNCHRONIZABLE (synchronizable));
   g_assert (EPHY_IS_SYNC_SERVICE (self));
 
+  monitor = g_network_monitor_get_default ();
+  if (g_network_monitor_get_connectivity (monitor) != G_NETWORK_CONNECTIVITY_FULL)
+    return;
+
   if (!ephy_sync_utils_user_is_signed_in ())
     return;
 
@@ -2206,10 +2220,16 @@ synchronizable_modified_cb (EphySynchronizableManager *manager,
                             gboolean                   should_force,
                             EphySyncService           *self)
 {
+  GNetworkMonitor *monitor;
+
   g_assert (EPHY_IS_SYNCHRONIZABLE_MANAGER (manager));
   g_assert (EPHY_IS_SYNCHRONIZABLE (synchronizable));
   g_assert (EPHY_IS_SYNC_SERVICE (self));
 
+  monitor = g_network_monitor_get_default ();
+  if (g_network_monitor_get_connectivity (monitor) != G_NETWORK_CONNECTIVITY_FULL)
+    return;
+
   if (!ephy_sync_utils_user_is_signed_in ())
     return;
 
@@ -2307,7 +2327,7 @@ ephy_sync_service_register_device (EphySyncService *self,
   json_object_set_string_member (record, "version", VERSION);
   json_object_set_array_member (record, "protocols", array);
   json_object_set_string_member (record, "os", "Linux");
-  json_object_set_null_member (record, "appPackage");
+  json_object_set_string_member (record, "appPackage", "org.gnome.epiphany");
   json_object_set_string_member (record, "application", "Epiphany");
 
   /* Get record's string representation. */
